@@ -12,16 +12,20 @@ def test_config(api):
     config.e.e_a = 1.1
     config.e.e_b = 1.2
     config.f.f_a = "a"
-    # g1 = config.g.append(name="unique list name", g_a="dkdkd", g_b=3, g_c=22.2)
-    # g1.g_d = "gdgdgd"
+    config.g.add(g_a="a g_a value")
     config.h = False
     config.i = "11011011"
-    # j = config.j.append()
-    # j.j_b.f_a = "a"
+    j1, j2 = config.j.jobject().jobject()
+    assert j1 == j2
+    config.k.e_object.e_a = 77.7
+    config.k.f_object.f_a = "asdf"
+    # assert j1.choice == j1._DEFAULTS["choice"]
     djson = json.loads(config.serialize(config.JSON))
     assert jp.parse("$.a").find(djson)[0].value == config.a
     assert jp.parse("$.f.f_a").find(djson)[0].value == config.f.f_a
-    print(config)
+    yaml = config.serialize(encoding=config.JSON)
+    config.deserialize(yaml)
+    print(yaml)
 
 
 if __name__ == "__main__":
